# vim: set fileencoding=utf-8 :
"""
Utilities for testing


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""   
from __future__ import with_statement
from django.test import Client
from django.core.handlers.wsgi import WSGIRequest
from django.contrib.sessions.middleware import SessionMiddleware

class MockRequestClient(Client):
    """
    A ``django.test.Client`` subclass which can return mock
    ``HttpRequest`` objects.
    
    """
    def request(self, **request):
        """
        Rather than issuing a request and returning the response, this
        simply constructs an ``HttpRequest`` object and returns it.
        
        """
        environ = {
            'HTTP_COOKIE': self.cookies,
            'PATH_INFO': '/',
            'QUERY_STRING': '',
            'REMOTE_ADDR': '127.0.0.1',
            'REQUEST_METHOD': 'GET',
            'SCRIPT_NAME': '',
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': '80',
            'SERVER_PROTOCOL': 'HTTP/1.1',
            'wsgi.version': (1,0),
            'wsgi.url_scheme': 'http',
            'wsgi.errors': self.errors,
            'wsgi.multiprocess':True,
            'wsgi.multithread': False,
            'wsgi.run_once': False,
            'wsgi.input': None,
            }
        environ.update(self.defaults)
        environ.update(request)
        request = WSGIRequest(environ)

        # We have to manually add a session since we'll be bypassing
        # the middleware chain.
        session_middleware = SessionMiddleware()
        session_middleware.process_request(request)
        return request


def mock_request():
    """
    Construct and return a mock ``HttpRequest`` object; this is used
    in testing backend methods which expect an ``HttpRequest`` but
    which are not being called from views.
    
    """
    return MockRequestClient().request()

def create_user(username):
    from django.contrib.auth.models import User
    user = User.objects.create_user(
            username=username,
            email='%s@test.com'%username,
            password='password'
        )
    return user

def create_role(name, parent=None):
    from permission.models import Role
    role = Role.objects.create(
            name=name, codename=name, description=name,
            parent=parent,
        )
    return role

def create_permission(name):
    from django.contrib.auth.models import Permission
    from django.contrib.contenttypes.models import ContentType
    from permission.models import Role
    ct = ContentType.objects.get_for_model(Role)
    permission = Permission.objects.create(
            name=name, codename=name,
            content_type=ct
        )
    return permission



# vim: set fileencoding=utf-8 :
"""
Permission handler base


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.contrib.auth.models import Permission

__all__ = ('PermissionHandler',)


class PermissionHandler(object):
    """Permission handler base class.

    You must create subclasses of this to create your custom permission handler.
    And you MUST DEFINE ``has_perm(user_obj, perm, obj=None)`` method to check
    the permission of passed ``user_obj``

    ``get_permissions()`` method is used to determine which permission strings are
    should be handled with this handler. In default, this method return the value
    of ``permissions`` attribute if defined. Otherwise it return the same value as
    ``get_model_permissions()`` method which described below.

    ``get_model_permissions()`` method return permission strings which is related
    to the ``model`` registered with this handler instance. ``get_app_permissions()``
    method return permission strings which is related to the app of the ``model``
    registered with this handler instance.

    ``get_permission_codename(perm)`` method return codename of ``perm``.

    """
    def __init__(self, model):
        self.model = model

    def get_app_permissions(self):
        """get permissions associated with app of this model"""
        if not hasattr(self, '_app_perm_cache'):
            app_label = self.model._meta.app_label
            qs = Permission.objects.filter(content_type__app_label=app_label)
            app_perms = set([u"%s.%s" % (app_label, p.codename) for p in qs.iterator()])
            self._app_perm_cache = app_perms
        return set(self._app_perm_cache)

    def get_model_permissions(self):
        """get permissions associated with this model"""
        if not hasattr(self, '_model_perm_cache'):
            app_label = self.model._meta.app_label
            model = self.model._meta.object_name.lower()
            qs = Permission.objects.filter(
                        content_type__app_label=app_label,
                        content_type__model=model
                    )
            model_perms = set([u"%s.%s" % (app_label, p.codename) for p in qs.iterator()])
            self._model_perm_cache = model_perms
        return set(self._model_perm_cache)

    def get_permissions(self):
        """get permissions which this handler will handle"""
        if not hasattr(self, 'permissions'):
            # build permission list associated with the model
            self.permissions = self.get_model_permissions()
        return set(self.permissions)

    def get_permission_codename(self, perm):
        """get permission codename from permission string"""
        from permission.utils import get_permission_codename
        return get_permission_codename(perm)

    def has_perm(self, user_obj, perm, obj=None):
        """whether the ``user_obj`` has permission ``perm`` of ``obj``

        This method will be called in ``has_perm()`` method of authentication
        backend so you CAN NOT call ``user_obj.has_perm()`` method within
        this method.

        Subclasses must override this method to define them own checking process.

        """
        raise NotImplementedError(
                '"%s" does not have ``has_perm`` method. Subclasses must '
                'define ``has_perm(user_obj, perm, obj=None)`` method.'
            )

    def has_module_perm(self, user_obj, app_label):
        """whether the ``user_obj`` has permissions of ``app_label``"""
        return False



import warnings
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from utils import autodiscover
from handlers import registry
from handlers import PermissionHandler

__all__ = ('autodiscover', 'registry', 'PermissionHandler')

installed = 'permission' in settings.INSTALLED_APPS

# set default settings
def setconf(name, default_value):
    value = getattr(settings, name, default_value)
    setattr(settings, name, value)

setconf('PERMISSION_MODULE_NAME', 'permissions')
setconf('PERMISSION_BUILTIN_TEMPLATETAGS', True)
setconf('PERMISSION_REPLACE_BUILTIN_IF', True)
setconf('PERMISSION_EXTEND_USER_CLASS', True)

# validate settings
if installed:
    if 'django.contrib.auth' \
            not in settings.INSTALLED_APPS:
        raise ImproperlyConfigured(
                '"django.contrib.auth" is not found in '
                '`INSTALLED_APPS`. You need to use "permission" '
                'with "django.contrib.auth".'
            )
    if 'permission.backends.PermissionBackend' \
            not in settings.AUTHENTICATION_BACKENDS:
        warnings.warn(Warning,
                '"permission.backends.PermissionBackend" is not found in '
                '`AUTHENTICATION_BACKENDS`.'
            )
    if 'permission.backends.RoleBackend' \
            not in settings.AUTHENTICATION_BACKENDS:
        warnings.warn(Warning,
                '"permission.backends.RoleBackend" is not found in '
                '`AUTHENTICATION_BACKENDS`.'
            )

# Register templatetags in builtin
if settings.PERMISSION_BUILTIN_TEMPLATETAGS:
    from django.template import add_to_builtins
    add_to_builtins('permission.templatetags.permission_tags')

# Extend User class
if settings.PERMISSION_EXTEND_USER_CLASS:
    from django.contrib import auth
    from django.contrib.auth.models import User
    from django.contrib.auth.models import AnonymousUser
    def _user_has_role(user, role):
        anon = user.is_anonymous()
        active = user.is_active
        for backend in auth.get_backends():
            if anon or active or backend.supports_inactive_user:
                if hasattr(backend, 'has_role'):
                    if backend.has_role(user, role):
                        return True
        return False
    def _user_get_all_roles(user):
        anon = user.is_anonymous()
        active = user.is_active
        for backend in auth.get_backends():
            if anon or active or backend.supports_inactive_user:
                if hasattr(backend, 'get_all_roles'):
                    return backend.get_all_roles(user)
        return None
    User.has_role = _user_has_role
    User.roles = property(_user_get_all_roles)
    AnonymousUser.has_role = lambda user, role: False
    AnonymousUser.roles = ()

# Auto-discover INSTALLED_APPS permissions.py modules
if installed:
    autodiscover()



# vim: set fileencoding=utf-8 :
"""
Unittest module of ...


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.test import TestCase

from permission.models import Role
from permission.tests.utils import create_user
from permission.tests.utils import create_role
from permission.tests.utils import create_permission


class PermissionRoleManagerTestCase(TestCase):

    def test_filter_by_user(self):
        # role1           -- user1, user2
        #   +- role2      -- user3
        #   +- role3      -- user4, user5
        #   |    +- role4 -- user6
        #   |    +- role5 -- user7
        #   +- role6      -- user8
        user1 = create_user('permission_test_user1')
        user2 = create_user('permission_test_user2')
        user3 = create_user('permission_test_user3')
        user4 = create_user('permission_test_user4')
        user5 = create_user('permission_test_user5')
        user6 = create_user('permission_test_user6')
        user7 = create_user('permission_test_user7')
        user8 = create_user('permission_test_user8')
        role1 = create_role('permission_test_role1')
        role2 = create_role('permission_test_role2', role1)
        role3 = create_role('permission_test_role3', role1)
        role4 = create_role('permission_test_role4', role3)
        role5 = create_role('permission_test_role5', role3)
        role6 = create_role('permission_test_role6', role1)
        role1._users.add(user1, user2)
        role2._users.add(user3)
        role3._users.add(user4, user5)
        role4._users.add(user6)
        role5._users.add(user7)
        role6._users.add(user8)

        self.assertItemsEqual(Role.objects.filter_by_user(user1), [
                role1,
            ])
        self.assertItemsEqual(Role.objects.filter_by_user(user2), [
                role1,
            ])
        self.assertItemsEqual(Role.objects.filter_by_user(user3), [
                role1, role2,
            ])
        self.assertItemsEqual(Role.objects.filter_by_user(user4), [
                role1, role3,
            ])
        self.assertItemsEqual(Role.objects.filter_by_user(user5), [
                role1, role3,
            ])
        self.assertItemsEqual(Role.objects.filter_by_user(user6), [
                role1, role3, role4,
            ])
        self.assertItemsEqual(Role.objects.filter_by_user(user7), [
                role1, role3, role5,
            ])
        self.assertItemsEqual(Role.objects.filter_by_user(user8), [
                role1, role6,
            ])

    def test_get_all_permissions_of_user(self):
        # role1           -- user1, user2 -- perm1, perm2
        #   +- role2      -- user3        -- perm3
        #   +- role3      -- user4, user5 -- perm4, perm5
        #   |    +- role4 -- user6        -- perm6
        #   |    +- role5 -- user7        -- perm7
        #   +- role6      -- user8        -- perm8
        user1 = create_user('permission_test_user1')
        user2 = create_user('permission_test_user2')
        user3 = create_user('permission_test_user3')
        user4 = create_user('permission_test_user4')
        user5 = create_user('permission_test_user5')
        user6 = create_user('permission_test_user6')
        user7 = create_user('permission_test_user7')
        user8 = create_user('permission_test_user8')
        perm1 = create_permission('permission_test_perm1')
        perm2 = create_permission('permission_test_perm2')
        perm3 = create_permission('permission_test_perm3')
        perm4 = create_permission('permission_test_perm4')
        perm5 = create_permission('permission_test_perm5')
        perm6 = create_permission('permission_test_perm6')
        perm7 = create_permission('permission_test_perm7')
        perm8 = create_permission('permission_test_perm8')
        role1 = create_role('permission_test_role1')
        role2 = create_role('permission_test_role2', role1)
        role3 = create_role('permission_test_role3', role1)
        role4 = create_role('permission_test_role4', role3)
        role5 = create_role('permission_test_role5', role3)
        role6 = create_role('permission_test_role6', role1)
        role1._users.add(user1, user2)
        role2._users.add(user3)
        role3._users.add(user4, user5)
        role4._users.add(user6)
        role5._users.add(user7)
        role6._users.add(user8)
        role1._permissions.add(perm1, perm2)
        role2._permissions.add(perm3)
        role3._permissions.add(perm4, perm5)
        role4._permissions.add(perm6)
        role5._permissions.add(perm7)
        role6._permissions.add(perm8)

        self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user1), [
                perm1, perm2,
            ])
        self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user2), [
                perm1, perm2,
            ])
        self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user3), [
                perm1, perm2, perm3,
            ])
        self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user4), [
                perm1, perm2, perm4, perm5,
            ])
        self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user5), [
                perm1, perm2, perm4, perm5,
            ])
        self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user6), [
                perm1, perm2, perm4, perm5, perm6,
            ])
        self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user7), [
                perm1, perm2, perm4, perm5, perm7,
            ])
        self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user8), [
                perm1, perm2, perm8,
            ])



class PermissionRoleModelTestCase(TestCase):

    def testcreate(self):
        role = create_role('permission_test_role1')
        self.assertItemsEqual(role.name, 'permission_test_role1')
        self.assertItemsEqual(role.codename, 'permission_test_role1')
        self.assertItemsEqual(role.description, 'permission_test_role1')

        return role

    def test__get_all_users(self):
        # role1           -- user1, user2
        #   +- role2      -- user3
        #   +- role3      -- user4, user5
        #   |    +- role4 -- user6
        #   |    +- role5 -- user7
        #   +- role6      -- user8
        user1 = create_user('permission_test_user1')
        user2 = create_user('permission_test_user2')
        user3 = create_user('permission_test_user3')
        user4 = create_user('permission_test_user4')
        user5 = create_user('permission_test_user5')
        user6 = create_user('permission_test_user6')
        user7 = create_user('permission_test_user7')
        user8 = create_user('permission_test_user8')
        role1 = create_role('permission_test_role1')
        role2 = create_role('permission_test_role2', role1)
        role3 = create_role('permission_test_role3', role1)
        role4 = create_role('permission_test_role4', role3)
        role5 = create_role('permission_test_role5', role3)
        role6 = create_role('permission_test_role6', role1)
        role1._users.add(user1, user2)
        role2._users.add(user3)
        role3._users.add(user4, user5)
        role4._users.add(user6)
        role5._users.add(user7)
        role6._users.add(user8)

        self.assertItemsEqual(role1.users, [
                user1, user2, user3, user4,
                user5, user6, user7, user8,
            ])
        self.assertItemsEqual(role2.users, [
                user3,
            ])
        self.assertItemsEqual(role3.users, [
                user4, user5, user6, user7,
            ])
        self.assertItemsEqual(role4.users, [
                user6,
            ])
        self.assertItemsEqual(role5.users, [
                user7,
            ])
        self.assertItemsEqual(role6.users, [
                user8,
            ])

    def test__get_all_permissions(self):
        # role1           -- perm1, perm2
        #   +- role2      -- perm3
        #   +- role3      -- perm4, perm5
        #   |    +- role4 -- perm6
        #   |    +- role5 -- perm7
        #   +- role6      -- perm8
        perm1 = create_permission('permission_test_perm1')
        perm2 = create_permission('permission_test_perm2')
        perm3 = create_permission('permission_test_perm3')
        perm4 = create_permission('permission_test_perm4')
        perm5 = create_permission('permission_test_perm5')
        perm6 = create_permission('permission_test_perm6')
        perm7 = create_permission('permission_test_perm7')
        perm8 = create_permission('permission_test_perm8')
        role1 = create_role('permission_test_role1')
        role2 = create_role('permission_test_role2', role1)
        role3 = create_role('permission_test_role3', role1)
        role4 = create_role('permission_test_role4', role3)
        role5 = create_role('permission_test_role5', role3)
        role6 = create_role('permission_test_role6', role1)
        role1._permissions.add(perm1, perm2)
        role2._permissions.add(perm3)
        role3._permissions.add(perm4, perm5)
        role4._permissions.add(perm6)
        role5._permissions.add(perm7)
        role6._permissions.add(perm8)

        self.assertItemsEqual(role1.permissions, [
                perm1, perm2
            ])
        self.assertItemsEqual(role2.permissions, [
                perm1, perm2, perm3,
            ])
        self.assertItemsEqual(role3.permissions, [
                perm1, perm2, perm4, perm5,
            ])
        self.assertItemsEqual(role4.permissions, [
                perm1, perm2, perm4, perm5, perm6,
            ])
        self.assertItemsEqual(role5.permissions, [
                perm1, perm2, perm4, perm5, perm7,
            ])
        self.assertItemsEqual(role6.permissions, [
                perm1, perm2, perm8,
            ])

    def test_is_belong(self):
        # role1           -- user1, user2
        #   +- role2      -- user3
        #   +- role3      -- user4, user5
        #   |    +- role4 -- user6
        #   |    +- role5 -- user7
        #   +- role6      -- user8
        user1 = create_user('permission_test_user1')
        user2 = create_user('permission_test_user2')
        user3 = create_user('permission_test_user3')
        user4 = create_user('permission_test_user4')
        user5 = create_user('permission_test_user5')
        user6 = create_user('permission_test_user6')
        user7 = create_user('permission_test_user7')
        user8 = create_user('permission_test_user8')
        role1 = create_role('permission_test_role1')
        role2 = create_role('permission_test_role2', role1)
        role3 = create_role('permission_test_role3', role1)
        role4 = create_role('permission_test_role4', role3)
        role5 = create_role('permission_test_role5', role3)
        role6 = create_role('permission_test_role6', role1)
        role1._users.add(user1, user2)
        role2._users.add(user3)
        role3._users.add(user4, user5)
        role4._users.add(user6)
        role5._users.add(user7)
        role6._users.add(user8)

        self.assertTrue(role1.is_belong(user1))
        self.assertTrue(role1.is_belong(user2))
        self.assertTrue(role1.is_belong(user3))
        self.assertTrue(role1.is_belong(user4))
        self.assertTrue(role1.is_belong(user5))
        self.assertTrue(role1.is_belong(user6))
        self.assertTrue(role1.is_belong(user7))
        self.assertTrue(role1.is_belong(user8))

        self.assertFalse(role2.is_belong(user1))
        self.assertFalse(role2.is_belong(user2))
        self.assertTrue(role2.is_belong(user3))
        self.assertFalse(role2.is_belong(user4))
        self.assertFalse(role2.is_belong(user5))
        self.assertFalse(role2.is_belong(user6))
        self.assertFalse(role2.is_belong(user7))
        self.assertFalse(role2.is_belong(user8))

        self.assertFalse(role3.is_belong(user1))
        self.assertFalse(role3.is_belong(user2))
        self.assertFalse(role3.is_belong(user3))
        self.assertTrue(role3.is_belong(user4))
        self.assertTrue(role3.is_belong(user5))
        self.assertTrue(role3.is_belong(user6))
        self.assertTrue(role3.is_belong(user7))
        self.assertFalse(role3.is_belong(user8))

    def test_add_users(self):
        # role1           -- user1, user2
        #   +- role2      -- user3
        #   +- role3      -- user4, user5
        #   |    +- role4 -- user6
        #   |    +- role5 -- user7
        #   +- role6      -- user8
        user1 = create_user('permission_test_user1')
        user2 = create_user('permission_test_user2')
        user3 = create_user('permission_test_user3')
        user4 = create_user('permission_test_user4')
        user5 = create_user('permission_test_user5')
        user6 = create_user('permission_test_user6')
        user7 = create_user('permission_test_user7')
        user8 = create_user('permission_test_user8')
        role1 = create_role('permission_test_role1')
        role2 = create_role('permission_test_role2', role1)
        role3 = create_role('permission_test_role3', role1)
        role4 = create_role('permission_test_role4', role3)
        role5 = create_role('permission_test_role5', role3)
        role6 = create_role('permission_test_role6', role1)
        role1.users.add(user1, user2)
        role2.users.add(user3)
        role3.users.add(user4, user5)
        role4.users.add(user6)
        role5.users.add(user7)
        role6.users.add(user8)

        self.assertItemsEqual(role1._users.all(), [
                user1, user2,
            ])
        self.assertItemsEqual(role2._users.all(), [
                user3,
            ])
        self.assertItemsEqual(role3._users.all(), [
                user4, user5,
            ])
        self.assertItemsEqual(role4._users.all(), [
                user6,
            ])
        self.assertItemsEqual(role5._users.all(), [
                user7,
            ])
        self.assertItemsEqual(role6._users.all(), [
                user8,
            ])

    def test_remove_users(self):
        # role1           -- user1, user2
        #   +- role2      -- user3
        #   +- role3      -- user4, user5
        #   |    +- role4 -- user6
        #   |    +- role5 -- user7
        #   +- role6      -- user8
        user1 = create_user('permission_test_user1')
        user2 = create_user('permission_test_user2')
        user3 = create_user('permission_test_user3')
        user4 = create_user('permission_test_user4')
        user5 = create_user('permission_test_user5')
        user6 = create_user('permission_test_user6')
        user7 = create_user('permission_test_user7')
        user8 = create_user('permission_test_user8')
        role1 = create_role('permission_test_role1')
        role2 = create_role('permission_test_role2', role1)
        role3 = create_role('permission_test_role3', role1)
        role4 = create_role('permission_test_role4', role3)
        role5 = create_role('permission_test_role5', role3)
        role6 = create_role('permission_test_role6', role1)
        role1._users.add(user1, user2)
        role2._users.add(user3)
        role3._users.add(user4, user5)
        role4._users.add(user6)
        role5._users.add(user7)
        role6._users.add(user8)
        role1.users.remove(user1, user2)
        role2.users.remove(user3)
        role3.users.remove(user4, user5)
        role4.users.remove(user6)
        role5.users.remove(user7)
        role6.users.remove(user8)

        self.assertItemsEqual(role1._users.all(), [])
        self.assertItemsEqual(role2._users.all(), [])
        self.assertItemsEqual(role3._users.all(), [])
        self.assertItemsEqual(role4._users.all(), [])
        self.assertItemsEqual(role5._users.all(), [])
        self.assertItemsEqual(role6._users.all(), [])

    def test_clear_users(self):
        # role1           -- user1, user2
        #   +- role2      -- user3
        #   +- role3      -- user4, user5
        #   |    +- role4 -- user6
        #   |    +- role5 -- user7
        #   +- role6      -- user8
        user1 = create_user('permission_test_user1')
        user2 = create_user('permission_test_user2')
        user3 = create_user('permission_test_user3')
        user4 = create_user('permission_test_user4')
        user5 = create_user('permission_test_user5')
        user6 = create_user('permission_test_user6')
        user7 = create_user('permission_test_user7')
        user8 = create_user('permission_test_user8')
        role1 = create_role('permission_test_role1')
        role2 = create_role('permission_test_role2', role1)
        role3 = create_role('permission_test_role3', role1)
        role4 = create_role('permission_test_role4', role3)
        role5 = create_role('permission_test_role5', role3)
        role6 = create_role('permission_test_role6', role1)
        role1._users.add(user1, user2)
        role2._users.add(user3)
        role3._users.add(user4, user5)
        role4._users.add(user6)
        role5._users.add(user7)
        role6._users.add(user8)
        role1.users.clear()
        role2.users.clear()
        role3.users.clear()
        role4.users.clear()
        role5.users.clear()
        role6.users.clear()

        self.assertItemsEqual(role1._users.all(), [])
        self.assertItemsEqual(role2._users.all(), [])
        self.assertItemsEqual(role3._users.all(), [])
        self.assertItemsEqual(role4._users.all(), [])
        self.assertItemsEqual(role5._users.all(), [])
        self.assertItemsEqual(role6._users.all(), [])


    def test_add_permissions(self):
        # role1           -- perm1, perm2
        #   +- role2      -- perm3
        #   +- role3      -- perm4, perm5
        #   |    +- role4 -- perm6
        #   |    +- role5 -- perm7
        #   +- role6      -- perm8
        perm1 = create_permission('permission_test_perm1')
        perm2 = create_permission('permission_test_perm2')
        perm3 = create_permission('permission_test_perm3')
        perm4 = create_permission('permission_test_perm4')
        perm5 = create_permission('permission_test_perm5')
        perm6 = create_permission('permission_test_perm6')
        perm7 = create_permission('permission_test_perm7')
        perm8 = create_permission('permission_test_perm8')
        role1 = create_role('permission_test_role1')
        role2 = create_role('permission_test_role2', role1)
        role3 = create_role('permission_test_role3', role1)
        role4 = create_role('permission_test_role4', role3)
        role5 = create_role('permission_test_role5', role3)
        role6 = create_role('permission_test_role6', role1)
        role1.permissions.add(perm1, perm2)
        role2.permissions.add(perm3)
        role3.permissions.add(perm4, perm5)
        role4.permissions.add(perm6)
        role5.permissions.add(perm7)
        role6.permissions.add(perm8)

        self.assertItemsEqual(role1._permissions.all(), [
                perm1, perm2,
            ])
        self.assertItemsEqual(role2._permissions.all(), [
                perm3,
            ])
        self.assertItemsEqual(role3._permissions.all(), [
                perm4, perm5,
            ])
        self.assertItemsEqual(role4._permissions.all(), [
                perm6,
            ])
        self.assertItemsEqual(role5._permissions.all(), [
                perm7,
            ])
        self.assertItemsEqual(role6._permissions.all(), [
                perm8,
            ])


    def test_remove_permissions(self):
        # role1           -- perm1, perm2
        #   +- role2      -- perm3
        #   +- role3      -- perm4, perm5
        #   |    +- role4 -- perm6
        #   |    +- role5 -- perm7
        #   +- role6      -- perm8
        perm1 = create_permission('permission_test_perm1')
        perm2 = create_permission('permission_test_perm2')
        perm3 = create_permission('permission_test_perm3')
        perm4 = create_permission('permission_test_perm4')
        perm5 = create_permission('permission_test_perm5')
        perm6 = create_permission('permission_test_perm6')
        perm7 = create_permission('permission_test_perm7')
        perm8 = create_permission('permission_test_perm8')
        role1 = create_role('permission_test_role1')
        role2 = create_role('permission_test_role2', role1)
        role3 = create_role('permission_test_role3', role1)
        role4 = create_role('permission_test_role4', role3)
        role5 = create_role('permission_test_role5', role3)
        role6 = create_role('permission_test_role6', role1)
        role1._permissions.add(perm1, perm2)
        role2._permissions.add(perm3)
        role3._permissions.add(perm4, perm5)
        role4._permissions.add(perm6)
        role5._permissions.add(perm7)
        role6._permissions.add(perm8)
        role1.permissions.remove(perm1, perm2)
        role2.permissions.remove(perm3)
        role3.permissions.remove(perm4, perm5)
        role4.permissions.remove(perm6)
        role5.permissions.remove(perm7)
        role6.permissions.remove(perm8)

        self.assertItemsEqual(role1._permissions.all(), [])
        self.assertItemsEqual(role2._permissions.all(), [])
        self.assertItemsEqual(role3._permissions.all(), [])
        self.assertItemsEqual(role4._permissions.all(), [])
        self.assertItemsEqual(role5._permissions.all(), [])
        self.assertItemsEqual(role6._permissions.all(), [])

    def test_clear_permissions(self):
        # role1           -- perm1, perm2
        #   +- role2      -- perm3
        #   +- role3      -- perm4, perm5
        #   |    +- role4 -- perm6
        #   |    +- role5 -- perm7
        #   +- role6      -- perm8
        perm1 = create_permission('permission_test_perm1')
        perm2 = create_permission('permission_test_perm2')
        perm3 = create_permission('permission_test_perm3')
        perm4 = create_permission('permission_test_perm4')
        perm5 = create_permission('permission_test_perm5')
        perm6 = create_permission('permission_test_perm6')
        perm7 = create_permission('permission_test_perm7')
        perm8 = create_permission('permission_test_perm8')
        role1 = create_role('permission_test_role1')
        role2 = create_role('permission_test_role2', role1)
        role3 = create_role('permission_test_role3', role1)
        role4 = create_role('permission_test_role4', role3)
        role5 = create_role('permission_test_role5', role3)
        role6 = create_role('permission_test_role6', role1)
        role1._permissions.add(perm1, perm2)
        role2._permissions.add(perm3)
        role3._permissions.add(perm4, perm5)
        role4._permissions.add(perm6)
        role5._permissions.add(perm7)
        role6._permissions.add(perm8)
        role1.permissions.clear()
        role2.permissions.clear()
        role3.permissions.clear()
        role4.permissions.clear()
        role5.permissions.clear()
        role6.permissions.clear()

        self.assertItemsEqual(role1._permissions.all(), [])
        self.assertItemsEqual(role2._permissions.all(), [])
        self.assertItemsEqual(role3._permissions.all(), [])
        self.assertItemsEqual(role4._permissions.all(), [])
        self.assertItemsEqual(role5._permissions.all(), [])
        self.assertItemsEqual(role6._permissions.all(), [])



# vim: set fileencoding=utf-8 :
"""
Unittest module of ...


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.test import TestCase

from permission.handlers import registry
from permission.handlers import PermissionHandler
from permission.backends import PermissionBackend
from permission.tests.override_settings import with_apps
from permission.tests.override_settings import override_settings

class TestPermissionHandler(PermissionHandler):
    called = False
    def has_perm(self, user_obj, perm, obj=None):
        self.called = True
        if not user_obj.is_authenticated():
            # No anonymous user have all permissions
            return False
        codename = self.get_permission_codename(perm)
        if codename == 'add_article':
            # Authenticated user have add permission
            return True
        if obj and obj.author == user_obj:
            # Author have change/delete permission
            return True
        # Others doesn't
        return False

@with_apps('permission.tests.test_app')
class PermissionBackendTestCase(TestCase):

    fixtures = ('permission_test_app.yaml',)

    def setUp(self):
        from django.contrib.auth.models import User
        from django.contrib.auth.models import AnonymousUser
        from permission.tests.test_app.models import Article
        self.anonymous = AnonymousUser()
        # superuser
        self.user1 = User.objects.get(username='permission_test_user1')
        # staff
        self.user2 = User.objects.get(username='permission_test_user2')
        # normal
        self.user3 = User.objects.get(username='permission_test_user3')
        self.user4 = User.objects.get(username='permission_test_user4')
        # author = user3
        self.article1 = Article.objects.get(title='permission_test_article1')
        # author = user4
        self.article2 = Article.objects.get(title='permission_test_article2')

        # backup original registry
        self._original_registry = registry._registry
        self._original_permissions = registry._permissions
        registry._registry = {}
        registry._permissions = {}

    def tearDown(self):
        # restore original registry
        registry._registry = self._original_registry
        registry._permissions = self._original_permissions

    def test_attributes_required(self):
        backend = PermissionBackend()

        self.assertTrue(hasattr(backend, 'supports_object_permissions'))
        self.assertTrue(getattr(backend, 'supports_object_permissions'))

        self.assertTrue(hasattr(backend, 'supports_anonymous_user'))
        self.assertTrue(getattr(backend, 'supports_anonymous_user'))

        self.assertTrue(hasattr(backend, 'supports_inactive_user'))
        self.assertTrue(getattr(backend, 'supports_inactive_user'))

        self.assertTrue(hasattr(backend, 'authenticate'))
        self.assertEqual(backend.authenticate(None, None), None)

        self.assertTrue(hasattr(backend, 'has_perm'))

    def test_has_perm(self):
        from permission.tests.test_app.models import Article
        registry.register(Article, TestPermissionHandler)

        backend = PermissionBackend()

        # the registered handler is used
        backend.has_perm(self.anonymous, 'test_app.add_article')
        self.assertTrue(registry._registry[Article].called)

        # the handler treat 'add', 'chage', 'delete' permissions
        permissions = registry._registry[Article].get_permissions()
        self.assertTrue('test_app.add_article' in permissions)
        self.assertTrue('test_app.change_article' in permissions)
        self.assertTrue('test_app.delete_article' in permissions)

        # anonymous user have no permission
        self.assertFalse(backend.has_perm(self.anonymous, 'test_app.add_article'))
        self.assertFalse(backend.has_perm(
                self.anonymous, 'test_app.change_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.anonymous, 'test_app.delete_article', self.article1
            ))
        # superuser have all permission generally but in this backend, they 
        # don't. the permissions for superuser will handled by downstream.
        self.assertTrue(backend.has_perm(self.user1, 'test_app.add_article'))
        self.assertFalse(backend.has_perm(
                self.user1, 'test_app.change_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.user1, 'test_app.delete_article', self.article1
            ))
        # staff user
        self.assertTrue(backend.has_perm(self.user2, 'test_app.add_article'))
        self.assertFalse(backend.has_perm(
                self.user2, 'test_app.change_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.user2, 'test_app.delete_article', self.article1
            ))
        # user3 is an author of the article1 so have all permissions
        # but for article2
        self.assertTrue(backend.has_perm(self.user3, 'test_app.add_article'))
        self.assertTrue(backend.has_perm(
                self.user3, 'test_app.change_article', self.article1
            ))
        self.assertTrue(backend.has_perm(
                self.user3, 'test_app.delete_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.user3, 'test_app.change_article', self.article2
            ))
        self.assertFalse(backend.has_perm(
                self.user3, 'test_app.delete_article', self.article2
            ))
        # user4 is an author of the article2 so have all permissions
        # but for article1
        self.assertTrue(backend.has_perm(self.user4, 'test_app.add_article'))
        self.assertFalse(backend.has_perm(
                self.user4, 'test_app.change_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.user4, 'test_app.delete_article', self.article1
            ))
        self.assertTrue(backend.has_perm(
                self.user4, 'test_app.change_article', self.article2
            ))
        self.assertTrue(backend.has_perm(
                self.user4, 'test_app.delete_article', self.article2
            ))

    def test_has_perm_permissions(self):
        from permission.tests.test_app.models import Article
        class TestPermissionHandler2(TestPermissionHandler):
            # 'test_app.add_article' is removed so the handler is not used
            # for treating that permission.
            permissions = [
                'test_app.change_article', 
                'test_app.delete_article',
            ]
        registry.register(Article, TestPermissionHandler2)

        backend = PermissionBackend()

        # the handler treat 'chage', 'delete' permissions
        permissions = registry._registry[Article].get_permissions()
        self.assertFalse('test_app.add_article' in permissions)
        self.assertTrue('test_app.change_article' in permissions)
        self.assertTrue('test_app.delete_article' in permissions)

        # anonymous user have no permission
        self.assertFalse(backend.has_perm(self.anonymous, 'test_app.add_article'))
        self.assertFalse(backend.has_perm(
                self.anonymous, 'test_app.change_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.anonymous, 'test_app.delete_article', self.article1
            ))
        # superuser have all permission generally but in this backend, they 
        # don't. the permissions for superuser will handled by downstream.
        self.assertFalse(backend.has_perm(self.user1, 'test_app.add_article'))
        self.assertFalse(backend.has_perm(
                self.user1, 'test_app.change_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.user1, 'test_app.delete_article', self.article1
            ))
        # staff user
        self.assertFalse(backend.has_perm(self.user2, 'test_app.add_article'))
        self.assertFalse(backend.has_perm(
                self.user2, 'test_app.change_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.user2, 'test_app.delete_article', self.article1
            ))
        # user3 is an author of the article1 so have all permissions
        # but for article2
        self.assertFalse(backend.has_perm(self.user3, 'test_app.add_article'))
        self.assertTrue(backend.has_perm(
                self.user3, 'test_app.change_article', self.article1
            ))
        self.assertTrue(backend.has_perm(
                self.user3, 'test_app.delete_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.user3, 'test_app.change_article', self.article2
            ))
        self.assertFalse(backend.has_perm(
                self.user3, 'test_app.delete_article', self.article2
            ))
        # user4 is an author of the article2 so have all permissions
        # but for article1
        self.assertFalse(backend.has_perm(self.user4, 'test_app.add_article'))
        self.assertFalse(backend.has_perm(
                self.user4, 'test_app.change_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.user4, 'test_app.delete_article', self.article1
            ))
        self.assertTrue(backend.has_perm(
                self.user4, 'test_app.change_article', self.article2
            ))
        self.assertTrue(backend.has_perm(
                self.user4, 'test_app.delete_article', self.article2
            ))

    def test_has_perm_get_permissions(self):
        from permission.tests.test_app.models import Article
        class TestPermissionHandler2(TestPermissionHandler):
            # 'test_app.add_article' is removed so the handler is not used
            # for treating that permission.
            def get_permissions(self):
                return set(['test_app.change_article', 'test_app.delete_article',])
        registry.register(Article, TestPermissionHandler2)

        backend = PermissionBackend()

        # the handler treat 'chage', 'delete' permissions
        permissions = registry._registry[Article].get_permissions()
        self.assertFalse('test_app.add_article' in permissions)
        self.assertTrue('test_app.change_article' in permissions)
        self.assertTrue('test_app.delete_article' in permissions)

        # anonymous user have no permission
        self.assertFalse(backend.has_perm(self.anonymous, 'test_app.add_article'))
        self.assertFalse(backend.has_perm(
                self.anonymous, 'test_app.change_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.anonymous, 'test_app.delete_article', self.article1
            ))
        # superuser have all permission generally but in this backend, they 
        # don't. the permissions for superuser will handled by downstream.
        self.assertFalse(backend.has_perm(self.user1, 'test_app.add_article'))
        self.assertFalse(backend.has_perm(
                self.user1, 'test_app.change_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.user1, 'test_app.delete_article', self.article1
            ))
        # staff user
        self.assertFalse(backend.has_perm(self.user2, 'test_app.add_article'))
        self.assertFalse(backend.has_perm(
                self.user2, 'test_app.change_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.user2, 'test_app.delete_article', self.article1
            ))
        # user3 is an author of the article1 so have all permissions
        # but for article2
        self.assertFalse(backend.has_perm(self.user3, 'test_app.add_article'))
        self.assertTrue(backend.has_perm(
                self.user3, 'test_app.change_article', self.article1
            ))
        self.assertTrue(backend.has_perm(
                self.user3, 'test_app.delete_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.user3, 'test_app.change_article', self.article2
            ))
        self.assertFalse(backend.has_perm(
                self.user3, 'test_app.delete_article', self.article2
            ))
        # user4 is an author of the article2 so have all permissions
        # but for article1
        self.assertFalse(backend.has_perm(self.user4, 'test_app.add_article'))
        self.assertFalse(backend.has_perm(
                self.user4, 'test_app.change_article', self.article1
            ))
        self.assertFalse(backend.has_perm(
                self.user4, 'test_app.delete_article', self.article1
            ))
        self.assertTrue(backend.has_perm(
                self.user4, 'test_app.change_article', self.article2
            ))
        self.assertTrue(backend.has_perm(
                self.user4, 'test_app.delete_article', self.article2
            ))

    def test_auth_backend(self):
        from permission.tests.test_app.models import Article
        registry.register(Article, TestPermissionHandler)

        with override_settings(AUTHENTICATION_BACKENDS=(
                    'django.contrib.auth.backends.ModelBackend',
                    'permission.backends.PermissionBackend',
                )):

            # PermissionBackend is used
            def is_permission_backend_used():
                from django.contrib.auth import get_backends
                for backend in get_backends():
                    if isinstance(backend, PermissionBackend):
                        return True
                return False
            self.assertTrue(is_permission_backend_used())

            # the handler treat 'add', 'chage', 'delete' permissions
            permissions = registry._registry[Article].get_permissions()
            self.assertTrue('test_app.add_article' in permissions)
            self.assertTrue('test_app.change_article' in permissions)
            self.assertTrue('test_app.delete_article' in permissions)

            # the registered handler is used (user1 is superuser so cannot be
            # used for this)
            self.user2.has_perm('test_app.add_article')
            self.assertTrue(registry._registry[Article].called)

            # superuser have all permission with ModelBackend
            self.assertTrue(self.user1.has_perm('test_app.add_article'))
            self.assertTrue(self.user1.has_perm(
                    'test_app.change_article', self.article1
                ))
            self.assertTrue(self.user1.has_perm(
                    'test_app.delete_article', self.article1
                ))
            # staff
            self.assertTrue(self.user2.has_perm('test_app.add_article'))
            self.assertFalse(self.user2.has_perm(
                    'test_app.change_article', self.article1
                ))
            self.assertFalse(self.user2.has_perm(
                    'test_app.delete_article', self.article1
                ))
            # user3 have all permissions for article1 but article2
            self.assertTrue(self.user3.has_perm('test_app.add_article'))
            self.assertTrue(self.user3.has_perm(
                    'test_app.change_article', self.article1
                ))
            self.assertTrue(self.user3.has_perm(
                    'test_app.delete_article', self.article1
                ))
            self.assertFalse(self.user3.has_perm(
                    'test_app.change_article', self.article2
                ))
            self.assertFalse(self.user3.has_perm(
                    'test_app.delete_article', self.article2
                ))
            # user4 have all permissions for article2 but article1
            self.assertTrue(self.user4.has_perm('test_app.add_article'))
            self.assertFalse(self.user4.has_perm(
                    'test_app.change_article', self.article1
                ))
            self.assertFalse(self.user4.has_perm(
                    'test_app.delete_article', self.article1
                ))
            self.assertTrue(self.user4.has_perm(
                    'test_app.change_article', self.article2
                ))
            self.assertTrue(self.user4.has_perm(
                    'test_app.delete_article', self.article2
                ))

        # without AUTH backend, handler wount called
        with override_settings(AUTHENTICATION_BACKENDS=(
                    'django.contrib.auth.backends.ModelBackend',
                )):

            # PermissionBackend is not used
            def is_permission_backend_used():
                from django.contrib.auth import get_backends
                for backend in get_backends():
                    if isinstance(backend, PermissionBackend):
                        return True
                return False
            self.assertFalse(is_permission_backend_used())

            # superuser have all permission with ModelBackend
            self.assertTrue(self.user1.has_perm('test_app.add_article'))
            self.assertTrue(self.user1.has_perm(
                    'test_app.change_article', self.article1
                ))
            self.assertTrue(self.user1.has_perm(
                    'test_app.delete_article', self.article1
                ))
            # users have no permissions
            self.assertFalse(self.user2.has_perm('test_app.add_article'))
            self.assertFalse(self.user2.has_perm(
                    'test_app.change_article', self.article1
                ))
            self.assertFalse(self.user2.has_perm(
                    'test_app.delete_article', self.article1
                ))
            self.assertFalse(self.user3.has_perm('test_app.add_article'))
            self.assertFalse(self.user3.has_perm(
                    'test_app.change_article', self.article1
                ))
            self.assertFalse(self.user3.has_perm(
                    'test_app.delete_article', self.article1
                ))
            self.assertFalse(self.user3.has_perm(
                    'test_app.change_article', self.article2
                ))
            self.assertFalse(self.user3.has_perm(
                    'test_app.delete_article', self.article2
                ))
            self.assertFalse(self.user4.has_perm('test_app.add_article'))
            self.assertFalse(self.user4.has_perm(
                    'test_app.change_article', self.article1
                ))
            self.assertFalse(self.user4.has_perm(
                    'test_app.delete_article', self.article1
                ))
            self.assertFalse(self.user4.has_perm(
                    'test_app.change_article', self.article2
                ))
            self.assertFalse(self.user4.has_perm(
                    'test_app.delete_article', self.article2
                ))



# vim: set fileencoding=utf-8 :
"""
Authentication backends for checking permissions

Method:
    get_permission_handler
        get permission handler of the model from registry

Class:
    PermissionBackend
        Authentication backend for checking permissions


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement

from permission.handlers import registry

__all__ = ('PermissionBackend',)


class PermissionBackend(object):
    """Authentication backend for cheking permissions
    
    This backend is used to check permissions. The permissions
    are handled with ``PermissionHandler`` which have to be registered in
    ``fluidpermission.handlers.registry`` before use.

    ``has_perm(user_obj, perm, obj=None)`` method of detected model's
    ``PermissionHandler`` will be used to cheking process.

    If no model was detected or no handler was registered, this backend
    does not touch that permission and return ``None`` to pass the permission
    checking process to downstream backends.

    """
    supports_object_permissions = True
    supports_anonymous_user = True
    supports_inactive_user = True

    def authenticate(self, username, password):
        """This backend is only for checking permission"""
        return None

    def has_perm(self, user_obj, perm, obj=None):
        """check permission"""
        # get permission handlers fot this perm
        handlers = registry.get_handlers(perm)
        for handler in handlers:
            # check permissions of handler
            if perm in handler.get_permissions():
                if handler.has_perm(user_obj, perm, obj=obj):
                    return True
        # do not touch this permission
        return False

    def has_module_perms(self, user_obj, app_label):
        # get permission handlers fot this perm
        handlers = registry.get_module_handlers(app_label)
        for handler in handlers:
            if handler.has_module_perms(user_obj, app_label):
                return True
        return False



# vim: set fileencoding=utf-8 :
from setuptools import setup, find_packages

version = '0.3.6'

def read(filename):
    import os.path
    return open(os.path.join(os.path.dirname(__file__), filename)).read()

setup(
    name="django-permission",
    version=version,
    description = "A enhanced permission system which enable object permission",
    long_description=read('README.rst'),
    classifiers = [
        # http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',

    ],
    keywords = "django permisssion auth",
    author = "Alisue",
    author_email = "lambdalisue@hashnote.net",
    url=r"https://github.com/lambdalisue/django-permission",
    download_url = r"https://github.com/lambdalisue/django-permission/tarball/master",
    license = 'MIT',
    packages = find_packages(exclude=['tests']),
    include_package_data = True,
    zip_safe=False,
    install_requires=[
        'distribute',
        'setuptools-git',
        'django>=1.3',
        'django-mptt',
    ],
    test_suite='runtests.runtests',
    tests_require=[
        'PyYAML',
        'feincms',
    ],
)
